/*
 * Control_AQM1602.h
 *
 *  Created on: 2024/11/21
 *      Author: egaono-electronics
 */

#ifndef CONTROL_AQM1602_H_
#define CONTROL_AQM1602_H_

#include "Dev_Conf.h"

#include def_Header_HAL

#include "H_STM32F4_I2C_1.h"



//----------------------------------------
// AQM1602 の スレーブアドレス
//----------------------------------------
#define def_SlaveAddress_AQM1602  0x007C



//----------------------------------------------------------
// I2C初期化 : AQM1602
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
//              Handle I2C用パラメータの構造体のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Initialize_I2C_AQM1602(uint8_t *puint8_JobNum,
            struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx);



//----------------------------------------------------------
// AQM1602 の初期設定を行う。
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx :
//              Handle I2C用パラメータの構造体のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Initial_Setting_AQM1602(uint8_t *puint8_JobNum,
            struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx);



//----------------------------------------------------------
// AQM1602 コマンド書き込み
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// uint8_t uint8_Number_TX : 送信データ数
// uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

// uint32_t uint32_Interval : 待ち時間 (mSec)

// struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx :
//              Handle I2C用パラメータの構造体のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Write_Command_AQM1602(uint8_t *puint8_JobNum,
                              uint8_t uint8_Number_TX,
                              uint8_t *puint8_aData_TX,
                              uint32_t uint32_Interval,
            struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx);



//----------------------------------------------------------
// AQM1602 データ書き込み
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// uint8_t uint8_Address : データ書き込みアドレス

// uint8_t uint8_Number_TX : 送信データ数
// uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

// struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx :
//              Handle I2C用パラメータの構造体のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Write_Data_AQM1602(uint8_t *puint8_JobNum,
                           uint8_t uint8_Address,
                           uint8_t uint8_Number_TX,
                           uint8_t *puint8_aData_TX,
            struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx);



//----------------------------------------------------------
// LCD AQM1602 のビジーチェック
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK
//    1 : NG
//----------------------------------------------------------
int16_t Check_Busy_AQM1602(uint8_t *puint8_JobNum);


#endif /* CONT_AQM1602_H_ */
