/*
 * Test_AQM1602.c
 *
 *  Created on: 2024/11/21
 *      Author: egaono-electronics
 */

#include "Test_AQM1602.h"

#include "Control_AQM1602.h"

#include "Time_Interval.h"


#include <string.h>


//----------------------------------------------------------
// LCD AQM1602 の表示試験
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx :
//              Handle I2C用パラメータの構造体のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK
//    1 : NG
//----------------------------------------------------------
int16_t Test_AQM1602(uint8_t *puint8_JobNum,
            struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx)
{
  //----------------------------------------------------
  // 呼び出す関数の引数に使用する処理番号は、
  // 必ず static の変数宣言をしてください。
  //----------------------------------------------------
  static uint8_t STC_uint8_JobNum_Time;  // Time_Interval処理番号

  //--------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言は、
  // 使用する関数内で、必ず static で宣言してください。
  //--------------------------------------------------------
  // Time_Interval用パラメータの構造体の宣言
  //--------------------------------------------------------
  static struct sParam_Time_Interval STC_structParam_Time_Interval;


  static uint8_t STC_uint8_JobNum_LCD;  // LCD処理番号

  static uint8_t STC_uint8_Address;       // 設定レジスタAddress
  static uint8_t STC_uint8_Number_TX;     // 送信データ数
  static uint8_t STC_uint8_aData_TX[16];  // 送信データを格納する配列


  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    //----------------------------------------------------------
    // Handle I2C用パラメータの構造体を初期化
    //----------------------------------------------------------
    // 引数：
    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //    なし
    //----------------------------------------------------------
    Initialize_Parameter_I2Cx(pstructParam_Handle_I2Cx);


    // 送信Bufferのポインタをセット
    pstructParam_Handle_I2Cx->puint8_aI2C_TxBuffer
                                = GLB_uint8_aI2C1_TxBuffer;

    // 受信Bufferのポインタをセット
    pstructParam_Handle_I2Cx->puint8_aI2C_RxBuffer
                                = GLB_uint8_aI2C1_RxBuffer;


    //----------------------------------------------------------
    // Addressing Mode指定
    //----------------------------------------------------------
    // AQM1602 の Addressing Mode
    // Specifies if 7-bit or 10-bit addressing mode is selected.
    //                        7bit : I2C_ADDRESSINGMODE_7BIT
    //                       10bit : I2C_ADDRESSINGMODE_10BIT
    pstructParam_Handle_I2Cx->uint32_AddressingMode
                        = (uint32_t)I2C_ADDRESSINGMODE_7BIT;
    //----------------------------------------------------------


    //----------------------------------------------------------
    // Slave Address指定
    //----------------------------------------------------------
    // AQM1602 の Slave Address
    pstructParam_Handle_I2Cx->uint16_vI2C_SlaveAddress
                        = (uint16_t)def_SlaveAddress_AQM1602;
    //----------------------------------------------------------


    //----------------------------------------------------------
    // Clock Speed指定
    //----------------------------------------------------------
    // AQM1602 の Clock Speed
    pstructParam_Handle_I2Cx->uint32_vI2C_Clock_Speed
                        = (uint32_t)def_I2C_CLOCK_SPEED_100k;
    //----------------------------------------------------------


    //----------------------------------------------------------
    // I2C初期化 : AQM1602
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Initialize_I2C_AQM1602(&STC_uint8_JobNum_LCD,
                                          pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_JobNum_LCD = 0;  // LCD表示処理番号初期化


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //----------------------------------------------------------
    // AQM1602 の初期設定を行う。
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx :
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Initial_Setting_AQM1602(&STC_uint8_JobNum_LCD,
                                           pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_Address = 0x80;  // レジスタアドレスをセット

    // LCD表示データをセット
    STC_uint8_Number_TX = 16;
    memcpy(STC_uint8_aData_TX, "0123456789ABCDEF", 16);


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //----------------------------------------------------------
    // AQM1602 データ書き込み
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // uint8_t uint8_Address : データ書き込みアドレス

    // uint8_t uint8_Number_TX : 送信データ数
    // uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Write_Data_AQM1602(&STC_uint8_JobNum_LCD,
                                      STC_uint8_Address,
                                      STC_uint8_Number_TX,
                                      STC_uint8_aData_TX,
                                      pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化
    STC_structParam_Time_Interval.uint32_Interval = (uint32_t)1000;
                                // 待ち時間セット : 1000mSec


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 3:
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == -1){
      // 処理中 : 時間未経過
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    STC_uint8_Address = 0xC0;

    STC_uint8_Number_TX = 16;
    memcpy(STC_uint8_aData_TX, "GHIJKLMNOPQRSTUV", 16);


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 4:
    //-------------------------------------------------------
    // AQM1602 データ書き込み
    //-------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // uint8_t uint8_Address : データ書き込みアドレス

    // uint8_t uint8_Number_TX : 送信データ数
    // uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //-------------------------------------------------------
    int16_Return = Write_Data_AQM1602(&STC_uint8_JobNum_LCD,
                                      STC_uint8_Address,
                                      STC_uint8_Number_TX,
                                      STC_uint8_aData_TX,
                                      pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化
    STC_structParam_Time_Interval.uint32_Interval = (uint32_t)1000;
                                // 待ち時間セット : 1000mSec


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 5:
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == -1){
      // 処理中 : 時間未経過
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    STC_uint8_Number_TX = 1;  // 送信データ数セット

    // コマンドセット
    STC_uint8_aData_TX[0] = 0x01;  // CLEAR DISPLAY


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 6:
    //----------------------------------------------------------
    // AQM1602 コマンド書き込み
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // uint8_t uint8_Number_TX : 送信データ数
    // uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

    // uint32_t uint32_Interval : 待ち時間 (mSec)

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Write_Command_AQM1602(&STC_uint8_JobNum_LCD,
                                         STC_uint8_Number_TX,
                                         STC_uint8_aData_TX,
                                         (uint32_t)3,
                                         pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_Address = 0x80;

    STC_uint8_Number_TX = 16;
    memcpy(STC_uint8_aData_TX, "Nice to meet you", 16);


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 7:
    //----------------------------------------------------------
    // AQM1602 データ書き込み
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // uint8_t uint8_Address : データ書き込みアドレス

    // uint8_t uint8_Number_TX : 送信データ数
    // uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Write_Data_AQM1602(&STC_uint8_JobNum_LCD,
                                      STC_uint8_Address,
                                      STC_uint8_Number_TX,
                                      STC_uint8_aData_TX,
                                      pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化
    STC_structParam_Time_Interval.uint32_Interval = (uint32_t)1000;
                                    // 待ち時間セット : 1000mSec


    (*puint8_JobNum)++;  // 次の処理番号に移行する。 : case 8:
    int16_Return = -1;   // 処理継続
    break;

  case 8:
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == -1){
      // 処理中 : 時間未経過
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    STC_uint8_Address = 0xC0;

    STC_uint8_Number_TX = 16;
    memcpy(STC_uint8_aData_TX, "Take it easy!   ", 16);


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 9:
    //----------------------------------------------------------
    // AQM1602 データ書き込み
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // uint8_t uint8_Address : データ書き込みアドレス

    // uint8_t uint8_Number_TX : 送信データ数
    // uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Write_Data_AQM1602(&STC_uint8_JobNum_LCD,
                                      STC_uint8_Address,
                                      STC_uint8_Number_TX,
                                      STC_uint8_aData_TX,
                                      pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_JobNum_Time = 0;  // Time_Interval処理番号初期化
    STC_structParam_Time_Interval.uint32_Interval = (uint32_t)1000;
                                    // 待ち時間セット : 1000mSec


    (*puint8_JobNum)++;  // 次の処理番号に移行する。 : case 10:
    int16_Return = -1;   // 処理継続
    break;

  case 10:
    //----------------------------------------------------------
    // 時間待ち : 単位 mSec
    //----------------------------------------------------------
    // 戻り値が -1 以外になるまで、繰り返し呼び出してください。
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Time_Interval *pstructParam_Time_Interval :
    //              Time_Interval用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中 : 時間が経過していない。
    //    0 : OK終了 : 時間が経過した。
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Time_Interval(&STC_uint8_JobNum_Time,
                                 &STC_structParam_Time_Interval);
    if(int16_Return == -1){
      // 処理中 : 時間未経過
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    STC_uint8_Number_TX = 1;  // 送信データ数セット

    // コマンドセット
    STC_uint8_aData_TX[0] = 0x01;  // CLEAR DISPLAY


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 11:
    //----------------------------------------------------------
    // AQM1602 コマンド書き込み
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // uint8_t uint8_Number_TX : 送信データ数
    // uint8_t *puint8_aData_TX : 送信データを格納するバッファのポインタ

    // uint32_t uint32_Interval : 待ち時間 (mSec)

    // struct sParam_Handle_I2Cx  *pstructParam_Handle_I2Cx
    //              Handle I2C用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Write_Command_AQM1602(&STC_uint8_JobNum_LCD,
                                         STC_uint8_Number_TX,
                                         STC_uint8_aData_TX,
                                         (uint32_t)3,
                                         pstructParam_Handle_I2Cx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      // NG終了
      HAL_I2C_DeInit(&Handle_I2C1);
      pstructParam_Handle_I2Cx->int16_vI2C_Initialize_Done = 0;

      *puint8_JobNum = 0;  // 処理番号初期化
      // int16_Return : 結果
      break;
    }


    STC_uint8_Address = 0x80;

    STC_uint8_Number_TX = 16;
    memcpy(STC_uint8_aData_TX, "0123456789ABCDEF", 16);


    *puint8_JobNum = 2;  // 処理番号 2 に移行する。: case 2:
                         // case 2: ～ case 11: をループする。
    int16_Return = -1;   // 処理継続
    break;

  default:
    *puint8_JobNum = 0;
    int16_Return = 1;
    break;  // NG
  }

  return(int16_Return);
}
